# Limitations : 
#   - Il faut utiliser \input{} et \include{} purement, et pas  l'interieur de
#     macros latex personnelles (ou la gestion des dpendances ne verra pas 
#     l'inclusion)
#   - Mme remarque  propos de \psfig{} et \includegraphics{}
#   - A propos du paquet graphics, \graphicspath est compris et gr (par un
#     vpath de make), mais pas \DeclareGraphicsExtensions (va falloir taper
#     '.eps' ou autre  chaque fois. Duurrr), ni \DeclareGraphicsRule{}{}{}{}.

THISDIR=$(shell basename `pwd`)
####[ Partie de configuration ]################################################

#VERB=normal  # silent quiet normal verbose

LATEX ?= latex
PDFLATEX ?= pdflatex
DVIPS ?= dvips
BIBTEX ?= bibtex
MPOST ?= TEX="$(LATEX)" mpost
FIG2DEV ?= fig2dev

WHICH ?= which 2>/dev/null
GENSUBFIG ?= $(word 1, $(wildcard bin/gensubfig ./gensubfig $(shell $(WHICH) gensubfig)))
FIGDEPTH ?= $(word 1, $(wildcard bin/figdepth ./figdepth $(shell $(WHICH) figdepth)))
TEXDEPENDS ?= $(word 1, $(wildcard bin/texdepends ./texdepends $(shell $(WHICH) texdepends)))
LATEXFILTER ?= $(word 1, $(wildcard bin/latexfilter ./latexfilter $(shell $(WHICH) latexfilter)))
EPSTOPDF ?= epstopdf

####[ Fin de la configuration ]################################################
# La modification de la suite est  vos risques et prils.

ifneq ($(strip $(BIBINPUTS)),)
vpath %.bib $(BIBINPUTS)
endif

ifneq ($(strip $(TEXINPUTS)),)
vpath %.tex $(TEXINPUTS)
endif

ifeq ($(FINAL),yes)
TEMPORAIRE=.INTERMEDIATE
else
TEMPORAIRE=.SECONDARY
endif

#---------------------------------------------------------------------
# Controle du niveau d'affichage
ifdef VERB
MAK_VERB :=  $(VERB)
else
#MAK_VERB :=  verbose
#MAK_VERB :=  normal
MAK_VERB :=  quiet
#MAK_VERB :=  silent
endif

#---------------------------------------------------------------------
# MAK_VERB -> niveau d'affichage des messages
ifeq ($(MAK_VERB),verbose)
COMMON_PREFIX  =  echo "         ======> building " $@ "<======" ;
#
COMMON_HIDE   :=#
COMMON_CLEAN  :=#
SHOW_LATEX:=true
else
ifeq ($(MAK_VERB),normal)
COMMON_PREFIX  =#
COMMON_HIDE   :=  @
COMMON_CLEAN  :=#
SHOW_LATEX:=true
else
ifeq ($(MAK_VERB),quiet)
COMMON_PREFIX  =  @ echo "         ======> building " $@ "<======" ;
#		echo "due to $?" ;
COMMON_HIDE   :=  @
COMMON_CLEAN  :=#
SHOW_LATEX:=
else  # silent
COMMON_PREFIX  =  @
COMMON_HIDE   :=  @
COMMON_CLEAN  :=  @
SHOW_LATEX:=
endif
endif
endif

#---------------------------------------------------------------------
# GOALS pour lesquels on ne gnrera ni n'incluera les .*.d
ifndef DO_NOT_GENERATE_DEP_FILES
DO_NOT_GENERATE_DEP_FILES :=
endif
DO_NOT_GENERATE_DEP_FILES += _clean_ _distclean_ _clean-logfile_


#---------------------------------------------------------------------
# La commande latex, avec son traitement d'erreur. Pour l'utiliser, faire:
#  		$(DO_LATEX) <cible> 
ifeq ($(strip $(SHOW_LATEX)),)
# excution en -interaction nonstopmode, on affiche que les erreurs
LATEX_OPT_PLUS=-interaction nonstopmode
LATEX_DISPLAY= >/dev/null
LATEX_DISPLAY_LOG=$(LATEXFILTER)
else
# excution en direct
LATEX_OPT_PLUS=
LATEX_DISPLAY=
LATEX_DISPLAY_LOG= :
endif
DO_LATEX = doit() {\
		case "$$2" in \
			*.dvi) CMD="$(LATEX)" ;; \
			*.pdf) CMD="$(PDFLATEX)" ;; \
			*) echo "Do no know how generate $$2"; exit 1 ;; \
		esac ; \
		echo "    ====> $$CMD $(LATEX_OPT) $(LATEX_OPT_PLUS) $$1"; \
		ret=0 ; \
		$$CMD $(LATEX_OPT) $(LATEX_OPT_PLUS) $$1 $(LATEX_DISPLAY) || ret=$$? ; \
		< $$1.log $(LATEXFILTER) || true; \
		if [ "$$ret" != 0 -a -z "$(SHOW_LATEX)" ]; then \
			cat $$1.log | awk '\
				BEGIN {keep=15; } \
				stop && keep && /^! / { keep=0;} \
				/^! / && !stop { stop=1; } \
				stop && keep { keep--} \
				keep { print}' ;\
			echo "$$CMD $(LATEX_OPT) $(LATEX_OPT_PLUS) $$1 [Aborted]" ;\
		fi ;\
		return $$ret; \
	}; \
	doit 

COMPILABLETEXFILES ?= $(shell grep -l '^[[:space:]]*\\documentclass' *.tex)
ALLTEXFILES ?= $(wildcard *.tex)

DVIFILES = $(subst .tex,.dvi,$(COMPILABLETEXFILES))
PSFILES = $(subst .tex,.ps,$(COMPILABLETEXFILES))
PSGZFILES = $(subst .ps,.ps.gz,$(PSFILES))
PDFFILES = $(subst .tex,.pdf,$(COMPILABLETEXFILES))
PDFGZFILES = $(subst .pdf,.pdf.gz,$(PDFFILES))
PSPDFFILES = $(subst .tex,.ps.pdf,$(COMPILABLETEXFILES))
PSPDFGZFILES = $(subst .pdf,.ps.pdf.gz,$(PDFFILES))

DEPFILES = $(patsubst %.tex,.%.d,$(COMPILABLETEXFILES))

TEXLOGFILES = $(foreach EXT, .log .aux .toc .lot .lof .out, \
	$(patsubst %.tex,%$(EXT),$(COMPILABLETEXFILES)))
BIBLOGFILES = $(foreach EXT, .bbl .blg, \
	$(patsubst %.tex,%$(EXT),$(COMPILABLETEXFILES)))
PDFLOGFILES = $(foreach EXT, .tpt,\
	$(patsubst %.tex,%$(EXT),$(COMPILABLETEXFILES)))
PSPDFLOGFILES = $(foreach EXT, .ps.tpm .pdf.ps, \
	$(patsubst %.tex,%$(EXT),$(COMPILABLETEXFILES)))

#FIGFIGURES = $(wildcard $(FIGDIR)/*.fig)
#FIGURES = $(wildcard fig/*.eps)
#GIFFIGURES = $(subst .eps,.gif,$(FIGURES))

ARCHIVEFILES=$(ALLTEXFILES) Makefile $(FIGURES)# $(FIGFIGURES) 
             # lib/intro.phtml lib/sed.tex2html lib/textohtml 

.PHONY: .figlatex
.figlatex:
	$(error "Please, define a default target before including this file")
#.PHONY: all
#all: index.html clean-logfile
#all:: clean-logfile
#all:: ps pdf clean

.PHONY: dvi pdf pdfgz ps psgz pspdf pspdfgz html
dvi: $(DVIFILES) 
pdf: $(PDFFILES) 
pdfgz: $(PDFGZFILES) clean-logfile
ps: $(PSFILES) 
psgz: $(PSGZFILES) clean-logfile
pspdf: $(PSPDFFILES) 
pspdfgz: $(PSPDFGZFILES) clean-logfile

html: $(HTMLFILES) $(GIFFIGURES)

####[ Les regles de nettoyage ]################################################
# Les fichiers  effacer sont ajouts prs des rgles qui les produisent

ifndef FILES_TO_CLEAN
FILES_TO_CLEAN :=
endif
ifndef FILES_TO_DISTCLEAN
FILES_TO_DISTCLEAN :=
endif

.PHONY: clean distclean

FILES_TO_CLEAN += $(wildcard *~)
clean::
	$(COMMON_HIDE)list=` \
		$(foreach file, $(sort $(FILES_TO_CLEAN)), \
			$(foreach dir, . $(subst :, ,$(VPATH)), \
				(test -f $(dir)/$(file) && \
					echo -n " $(dir)/$(file)" ) || \
			) true ;)` ; \
		echo "$(RM) $$list" ; \
		$(RM) $$list

distclean:: clean
	$(COMMON_CLEAN)$(RM) $(wildcard $(FILES_TO_DISTCLEAN))

ifeq (,$(findstring _$(MAKECMDGOALS)_,$(DO_NOT_GENERATE_DEP_FILES)))
.%.d: %.tex
ifeq ($(strip $(TEXDEPENDS)),)
	$(error 'texdepends' not found. Please, set $$TEXDEPENDS)
else
	$(COMMON_PREFIX)$(TEXDEPENDS) $* > $@
endif
endif

%.idx: 
	$(COMMON_PREFIX)$(DO_LATEX) $*

%.ind: %.idx
	-$(COMMON_PREFIX)`pwd`/$(LIBDIR)/index.pl $*

FILES_TO_DISTCLEAN += $(DVIFILES)
FILES_TO_CLEAN += $(TEXLOGFILES)
FILES_TO_CLEAN += $(PDFLOGFILES)
$(TEMPORAIRE): $(TEXLOGFILES) $(PDFLOGFILES)
FILES_TO_DISTCLEAN += $(PDFFILES)

%.pdf %.dvi: %.tex
	$(COMMON_PREFIX)set -e; \
	$(DO_LATEX) $* $@ ; \
	nb=`grep -c "Warning: Citation" $*.log` || true;\
	if [ $$nb -gt 0 ] ; then \
		echo "$$nb citations sont indfinies.";\
		$(MAKE) $*.bbl || exit -1; \
	fi; \
	nb=1; \
	if grep "No file.*[a][u][x]\\.$$" $*.log ; then \
		echo "Table des matires"; \
		$(DO_LATEX) $*  $@ ; \
		nb=2; \
	fi; \
	while grep "Rerun to get" $*.log ; do\
		echo "$$nb ieme compilation";\
		$(DO_LATEX) $* $@ ;\
		nb=`expr $$nb + 1`;\
		if [ "$$nb" = 5 ]; then \
			echo "     >>>>> Too many recompilations : stopping <<<<<" ; \
			break; \
		fi; \
	done


FILES_TO_CLEAN += $(BIBLOGFILES)
%.bbl:
	$(COMMON_PREFIX)$(BIBTEX) $*

PUBLISHED_RULE = \
	if [ -e $@ ] ; then
		echo "Cet article est publi. Pas de regnration inutile"; \
		touch $@; \
	else \
		echo "Cet article est publi. Regnration force"; \
		$(MAKE) PUBLISHED=no $(MAKECMDGOALS); \
	fi

ifeq ($(PUBLISHED),yes)
%.eps: %.fig
	$(COMMON_PREFIX)$(PUBLISHED_RULE)

%.pstex %.pstex_t %.pdf %.pdftex_t: %.fig
	$(COMMON_PREFIX)$(PUBLISHED_RULE)

else
%.eps: %.fig
	$(COMMON_PREFIX)$(FIG2DEV) -L eps $< $@

%.pstex: %.fig
	$(COMMON_PREFIX)$(FIG2DEV) -L pstex $< $@

%.pstex_t: %.fig
	$(COMMON_PREFIX)$(FIG2DEV) -L pstex_t -p $*.pstex $< $@

%.pdf: %.fig
	$(COMMON_PREFIX)$(FIG2DEV) -L pdftex $< $@

%.pdftex_t: %.fig
	$(COMMON_PREFIX)$(FIG2DEV) -L pdftex_t -p $*.pdf $< $@

endif

%.pdf: %.eps
	$(COMMON_PREFIX)$(EPSTOPDF) --filter < $< > $@

FILES_TO_DISTCLEAN += $(PSFILES)
$(PSFILES):	%.ps:	%.dvi
	$(COMMON_PREFIX)$(DVIPS) -o $@ $*
#@$(DVIPS) -d 9 -o $@ $* 2>&1 | grep Scanning | wc -l | sed 's/^/Nombre de pages: /g'

FILES_TO_CLEAN += $(PSPDFLOGFILES)
FILES_TO_DISTCLEAN += $(PSPDFFILES)
$(PSPDFFILES):	%.ps.pdf:	%.ps
	$(COMMON_PREFIX)ps2pdf $*.ps $@ >/dev/null 2>/dev/null
	$(COMMON_HIDE)echo "   Making thumbnails"
	$(COMMON_HIDE)thumbpdf -mode ps2pdf $@  2>/dev/null
	$(COMMON_HIDE)echo "   Incorporating them to the pdf"
	$(COMMON_HIDE)mv $*.dvi $*.dvi.keep
	$(COMMON_HIDE)$(DO_LATEX) $* $*.dvi
	$(COMMON_HIDE)$(DVIPS) -Ppdf $*.dvi -o $*.pdf.ps >/dev/null 2>/dev/null
	$(COMMON_HIDE)ps2pdf $*.pdf.ps $@ >/dev/null 2>/dev/null
	$(COMMON_HIDE)mv $*.dvi.keep $*.dvi

####[ Les regles pour gerer les subfigures ]###################################
FILES_TO_DISTCLEAN += .*_subfig.mk
.%_subfig.mk:
ifeq (,$(findstring _$(MAKECMDGOALS)_,$(DO_NOT_GENERATE_DEP_FILES)))
ifeq ($(strip $(GENSUBFIG)),)
	$(error 'gensubfig' not found. Please, set $$(GENSUBFIG))
endif
ifeq ($(strip $(FIGDEPTH)),)
	$(error 'figdepth' not found. Please, set $$(FIGDEPTH))
endif
	$(COMMON_PREFIX)$(GENSUBFIG) -p '$$(COMMON_PREFIX)$(FIGDEPTH) < $$< > $$@' \
		-s $^ $(dir $<)$*.fig < $^ > $@
else
	$(COMMON_HIDE) # fake regeneration of $@
%.subfig:
	$(COMMON_HIDE) # fake regeneration of $@
endif

####[ Les regles pour gerer les fichiers MetaPost ]############################
%.mp-stamp %.log %.mpx: %.mp
	$(COMMON_PREFIX)set -e ; cd $(dir $@); $(MPOST) $(basename $(notdir $@)) ; \
		for f in $(notdir $*)*; do \
			new=`echo $$f | sed -e s'/\.\([0-9][0-9]*\)$$/_\1.mps/'`; \
			test "$$f" != "$$new" && mv "$$f" "$$new" || true ;\
		done ; echo $* toto
	$(COMMON_HIDE)touch $@

%.mps:
	$(COMMON_PREFIX)echo "$@ already built with $<"

####[ Les regles pour gerer les archives ]#####################################
# TODO:  refaire...
.PHONY: FORCE extract archive

FILES_TO_DISTCLEAN += ../$(THISDIR).tgz.bak ../$(THISDIR).tgz

archive: ../$(THISDIR).tgz

extract:
	-tar xvzf ../$(THISDIR).tgz;

../$(THISDIR).tgz: $(ARCHIVEFILES)
	@rm -f $@.bak
	@if [ -e $@ ] ; then mv $@ $@.bak ; fi
	echo "Archivage dans $@"
	tar cvzf $@ $(ARCHIVEFILES)

####[ Regles pour faire du HTML ]##############################################
# TODO:  refaire...

FILES_TO_DISTCLEAN += index.html $(HTMLFILES) $(GIFFIGURES)

$(HTMLFILES):	%.html: %.tex lib/sed.tex2html lib/textohtml
	$(COMMON_PREFIX)./lib/textohtml $*.tex > $@

index.html: lib/duka Makefile $(GIFFIGURES)
	$(COMMON_PREFIX)touch $@
	$(COMMON_HIDE)-rm -f $@
	$(COMMON_HIDE)cat lib/intro.phtml  | sed "s/#date#/`date`/g" >> $@
	$(COMMON_HIDE)m="$(NONHTMLABLETEXFILES)"; if [ ! -z "$$m" ]; then for i in "$$m"; do j=`basename $$i .tex`; k=`grep '^\\\\session' $$i | sed -e 's/}//g'`; l=`echo $$k|cut -f4 -d{`; ll=`echo $$k|cut -f2 -d{`; if [ ! -z "$$l" ]; then echo "<li><a href=\"+++.ps.gz\">$$l</a> (++psgz++ ko)   <em><a href=\"+++.pdf\">PDF</a>(++pdf++ ko)|<a href=\"+++.dvi\">dvi</a>(++dvi++ ko)|<a href=\"+++.ps\">PS</a>(++ps++ ko)</em>"| sed -e "s/+++/$$j/g" -e "s/++pdf++/`grep [:space:]$$j.pdf$$ lib/duka|cut -f1`/g" -e "s/++dvi++/`grep [:space:]$$j.dvi$$ lib/duka|cut -f1`/g" -e "s/++ps++/`grep [:space:]$$j.ps$$ lib/duka|cut -f1`/g" -e "s/++psgz++/`grep [:space:]$$j.ps.gz$$ lib/duka|cut -f1`/g" >> $@; fi; done; fi
	$(COMMON_HIDE)m="$(HTMLABLETEXFILES)"; if [ ! -z "$$m" ]; then for i in $$m; do j=`basename $$i .tex`; k=`grep '^\\\\session' $$i | sed -e 's/}//g'`; l=`echo $$k|cut -f4 -d{`; ll=`echo $$k|cut -f2 -d{`; lll=`grep '^\\\\titre' $$i | sed -e 's/}//g' | sed -e 's/[^{]*{//g'`; if [ ! -z "$$lll" ]; then l="$$lll"; fi; if [ ! -z "$$l" ]; then echo "<li><a href=\"+++.html\">$$l</a> ($$ll) <em><a href=\"+++.ps.gz\">PS.gz</a>(++psgz++ ko)|<a href=\"+++.dvi\">dvi</a>(++dvi++ ko)|<a href=\"+++.ps\">PS</a>(++ps++ ko)</em>"| sed -e "s/+++/$$j/g" -e "s/++dvi++/`grep [:space:]$$j.dvi$$ lib/duka|cut -f1`/g" -e "s/++ps++/`grep [:space:]$$j.ps$$ lib/duka|cut -f1`/g" -e "s/++psgz++/`grep [:space:]$$j.ps.gz$$ lib/duka|cut -f1`/g" >> $@; fi; done; fi
	$(COMMON_HIDE)echo "<li><a href=\"gerard.tgz\">L'archive complte (### ko)</a> des fichiers LaTeX2e au format .tar.gz" | sed "s/###/`grep gerard.tgz lib/duka|cut -f1`/g">> $@
	$(COMMON_HIDE)echo "</UL></BODY></HTML>" >> $@

%.gif: %.eps
	$(COMMON_PREFIX)echo "showpage quit" | gs -q -sDEVICE=pnm -sOutputFile=- $*.eps | sed -e '1 s/.*/P3/g' | pnmcut `grep %%BoundingBox $*.eps | awk '{ print $$2,-$$5,$$4,$$5 }'` | ppmquant 240 | ppmtogif > $*.gif


####[ Inclusion des dpendances ]##############################################
FILES_TO_DISTCLEAN += $(DEPFILES) $(SUBFIGMAKEFILES)
ifeq (,$(findstring _$(MAKECMDGOALS)_,$(DO_NOT_GENERATE_DEP_FILES)))
ifneq ($(sort $(DEPFILES)),)
-include $(sort $(DEPFILES))
endif
ifneq ($(sort $(SUBFIGMAKEFILES)),)
-include $(sort $(SUBFIGMAKEFILES))
endif
else
ifneq ($(wildcard $(sort $(DEPFILES))),)
-include $(sort $(DEPFILES))
endif
ifneq ($(wildcard $(sort $(SUBFIGMAKEFILES))),)
-include $(sort $(SUBFIGMAKEFILES))
endif
endif

findfiles=$(foreach file, $(4) \
		$(foreach base,$(1), \
			$(wildcard $(base)) \
			$(foreach rep, $(subst :, ,$(VPATH)), \
				$(wildcard $(rep)$(base)))), \
	$(foreach ext,$(3), $(file:%$(2)=%.$(ext))))

FILES_FROM_FIGS:=$(call findfiles,$(sort $(FIGS_LIST)),.fig,\
	eps pstex pstex_t pdf pdftex_t, $(FIGS2CREATE_LIST))
FILES_FROM_MP:=$(call findfiles,$(sort $(MPS_LIST)),.mp,mp-stamp, )

FILES_TO_PDF:=$(call findfiles,$(sort $(PDFFROMEPS_LIST)),.eps,pdf, )

$(FILES_FROM_FIGS):
$(TEMPORAIRE): $(FILES_FROM_FIGS)

$(FILES_FROM_MP):
$(TEMPORAIRE): $(FILES_FROM_MP)

$(FILES_TO_PDF):
$(TEMPORAIRE): $(FILES_TO_PDF)

$(TEMPORAIRE):
