/*
 *  Copyright (C) 2003, 2004 Samuel Thibault <samuel.thibault@ens-lyon.org>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __FXT_TOOLS_H__
#define __FXT_TOOLS_H__
#include <stdint.h>
#include <errno.h>

#define EPRINTF(fmt,...)	do {\
	fprintf(stderr, "=== " fmt, ## __VA_ARGS__); \
	printf("=== " fmt, ## __VA_ARGS__); \
	fflush(stdout); \
} while (0)

#define DEFAULT_TRACE_FILE "trace_file"

#define WRITE(n,value) do { \
	int res; \
	uint##n##_t __val = (typeof(__val)) (value); \
	res = write(fd, (void *)&__val, sizeof(__val)); \
	if (res < 0) return -1; \
	if (res < sizeof(__val)) { \
		errno = EINVAL; \
		return -1; \
	} \
} while (0)

#define READ(n,value) do { \
	uint##n##_t __val; \
	int res; \
	res = read(fd, (void *)&__val, sizeof(__val)); \
	if (res < 0) return -1; \
	if (res < sizeof(__val)) { \
		errno = EINVAL; \
		return -1; \
	} \
	(value) = (typeof(value)) __val; \
} while (0)

/* Chaines pour le format des printf */
#ifdef FXT_ARCH_64
#  define ARCH_BITS 64
#  define L64 "l"
#  define L32 ""
#elif defined(FXT_ARCH_32)
#  define ARCH_BITS 32
#  define L64 "ll"
#  define L32 ""
#else
#  error no FXT_ARCH_32 nor FXT_ARCH_64 defined. Did you include <fxt.h> ?
#endif

#endif /* __FXT_TOOLS_H__ */
